/*
	File:		MoreOSA.cpp

	Contains:	C++ object for running OSA scripts

	Author:		Jon Pugh <mailto:jonpugh@frostbitefalls.com>

	Revisions:	7/19/99 First release
	
	Usage:		This reusable C++ object can be used pretty much as is.  It can
				load scripts from resources or memory, text or compiled.  It compiles,
				it loads, it stores, it runs, it returns results.  What more 
				do you want?
				
				This isn't heavily commented, but it was holding up the release,
				and since I'm doing this for free, the comments are sparse.

 */

#pragma once

#include <OSA.h>
#include <Files.h>
#include <Aliases.h>

class ScriptObject
{
	protected:
		ComponentInstance		fComponent;
		OSAID					fScriptID;
		OSAID					fResultID;
		Zone*					fZone;
		FSSpec					fFile;
		AEIdleUPP				fIdleProc;
		Boolean					fPersistentProperties;
		
	private:
		ScriptObject(const ScriptObject&);

	public:
		ScriptObject();
		~ScriptObject();
		
		// these will be performed automatically when needed		
		void OpenComponent();
		void CloseComponent();
		ComponentInstance GetComponent();
		
		// Memory control
		void UseZone(Zone* zone);
		void UseSystemZone();
		
		// use Compile or Load to initialize a script
		Boolean Compile(FSSpec *file);
		Boolean Compile(DescType type, Handle scriptText);
		
		// save the formatted script text
		void StoreSource();
		void StoreSource(FSSpec *file);
		
		// load a compiled script into memory
		void Load(FSSpec *file);
		void Load(AliasHandle alias);
		void Load(Handle scriptData);
		Boolean ScriptLoaded();
		
		// set a number of optional callbacks
		// idleProc should always be used or events will be lost
		void SetIdleProc(AEIdleUPP idleProc);
		// send and active procs provide time while scripts run
		void SetSendProc(OSASendUPP sendProc, long refcon);
		void SetActiveProc(OSAActiveUPP activeProc, long refcon);
		// these provide additional less used hooks
		void SetResumeProc(AEEventHandlerUPP resumeProc, long refcon);
		void SetCreateProc(OSACreateAppleEventUPP createProc, long refcon);
		
		// control whether Run methods call Store automatically
		Boolean GetPersistentProperties();
		void SetPersistentProperties(Boolean persist);
		
		// run a script by executing the run handler
		void Run();
		void Run(FSSpec *file);
		void Run(Handle scriptData);
		
		// execute an arbitrary handler
		void GetHandlerNames(AEDescList* handlers);
		Boolean HasHandler(char *handlerName);
		void CallSubroutine(char *handlerName, AEDescList *params);
		
		// send an event to the script
		void DoEvent(AppleEvent *event);
		void DoEvent(AppleEvent *event, AppleEvent *reply);
		
		// access info about the script
		Boolean IsModified();
		void GetSource(AEDesc *text);
		void GetSource(DescType desiredType, AEDesc *text);
		void ReturnResults(AEDesc *result);
		void ReturnResults(DescType desiredType, AEDesc *result);
		void ReturnResultText(AEDesc *result);
		void GetErrorMessage(DescType desiredType, AEDesc *result);
		long GetErrorNumber();
		void GetErrorRange(AEDesc *result);
		
		// save the original
		void Store();
		void Store(FSSpec *file);
		Handle Store(Handle oldScript);
		// save the original only if it changed
		void StoreIfChanged();
		void StoreIfChanged(FSSpec *file);
		Handle StoreIfChanged(Handle oldScript);
};

//	Error handling code
//
//	This AnyError class has only a constructor and these Macros.
//	It should be replacable with a minimum of effort.

class AnyError {
	public:
	OSAError	err;
	char		*msg;

	AnyError(OSAError e, char *m);
};


#define THROW_IF_ERR(x)	ThrowIfOSErr_(x)

#define THROW_IF_NIL(x)	ThrowIfNil_(x)

#define	TRY		try

#define CATCH	catch (const LException& e)

#define	RERAISE	throw(e)

#define	GETERROR (e.err)

